#ifndef _WAVENET_H_
#define _WAVENET_H_

#include "waveipx.h"

/**** Various limits ****/
#define MAX_SLOTS       8             // maximum players in a game
#define MAX_USERS       10            // maximum local users
#define NAME_SIZE       11            // maximum length of user names
#define CHAT_SIZE       80            // maximum length of chat lines
#define DATA_SIZE       100           // maximum data size of packets
#define IDLE_TICK       10            // seconds until timeout
#define PING_TICK       1             // seconds between ping packets

/**** User status defines ****/
#define WN_NONE    0                  // user does not exist
#define WN_LOGIN   1                  // user is in the login process
#define WN_IDLE    2                  // user has connected and is idle
#define WN_CLIENT  3                  // user is acting as a client
#define WN_SERVER  4                  // user is acting as a host

/**** Join status defines ****/
#define WN_JNONE      0               // user has no joins pending
#define WN_JPENDING   1               // user has not been notified yet
#define WN_JACCEPT    2               // user was allowed the join
#define WN_JREFUSE    3               // user was refused the join

/**** Wave.Net Server/Client Packets (1-15) ****/
#define WN_PLAYER_CONNECT          0  // player connecting to wavenet
#define WN_PLAYER_QUIT             1  // player joined the real world
#define WN_JOIN_REQUEST            2  // request to join a created game
#define WN_JOIN_ACCEPT             3  // permits player to join a game
#define WN_JOIN_REFUSE             4  // refuses player to join a game
#define WN_JOIN_INFORM             5  // informs other clients about join
#define WN_PLAYER_LEAVE            6  // informs game player has left
#define WN_REJECT_PLAYER           7  // disconnect by host (rejection)
#define WN_GAME_START              8  // game is about to begin
#define WN_PLAYER_CHAT             9  // chat line data

/**** Wave.Net Ingame Packets (15-30) ****/
#define WN_UPDATE                  15 // game update packet
#define WN_READY                   16 // ready to rock and roll
#define WN_LEAVE                   17 // quit game (not program)

/**** Wave.Net Admin Packets (30-45) ****/
#define WN_SERVER_SYNC             30 // server sync request
#define WN_CLIENT_SYNC             31 // client sync response
#define WN_PING                    32 // ping request
#define WN_PONG                    33 // ping response

/*
Game class, contains all information about a game.
*/
class GAME {
 public:
  char name[NAME_SIZE];               // game name (same as host user name)
  bool started;                       // true if the game has started
  int filled;                         // amount of filled slots
  NODEADD slot[MAX_SLOTS];            // each slot contains a node address
  unsigned char local[MAX_SLOTS];     // each slot has a local number attached
};

/*
User class, contains all information about a user.
*/
class USER {
 public:
  char name[NAME_SIZE];               // user name
  unsigned char local;                // local user number (internal use only)
  unsigned char status;               // status of user (see above for list)
  unsigned char player;               // player number (if in game)
  unsigned char skin;                 // player skin number (color/texture)
  unsigned char team;                 // player team number
  NODEADD node;                       // node address of user
  GAME game;                          // game associated with user
};

/*
WaveNet Packet class, all information passing through WaveNet must
use this packet format in the end (applied by low level send/get functions).
*/
class WNPACKET {
 public:
  char command;                       // packet command (see above for list)
  char info[DATA_SIZE];               // packet data
  USER user;                          // user that sent the packet
};

/*
Usernode class, node structure of the userlist.
*/
class USERNODE {
 public:
  USER user;                          // user information
  int tick;                           // ping timeout (internal use only)
  int pingtime;                       // ping timer (internal use only)
  float speed;                        // ping speed in milliseconds
  USERNODE *next;                     // next usernode in userlist
};

/*
Chatline class, node structure of the chatline list.
*/
class CHATLINE {
 public:
  char data[CHAT_SIZE];                // chat line string
  USERNODE *from;                      // user it came from
  CHATLINE *next;                      // next chatline in chatlist
};

/*
WaveNet class, handles the lobbying routines.
*/
class WAVENET {
 private:
  int tick;
  int users;
  int maxslots;
  bool online;
  USER user[MAX_USERS];
  int  join[MAX_USERS];
  USERNODE *userlist, *index;
  CHATLINE *chatlist;
 public:
  WAVENET();
  ~WAVENET();

  void set_max_slots( int max );
  int get_max_slots( );
  bool ipx_online( );
  USER *local_user( int u = 0 );
  int init_ipx( );
  void close_ipx( );
  int login( );
  void connect( char name[NAME_SIZE], int u = 0 );
  bool user_exists( char name[NAME_SIZE] );
  void disconnect( int u = 0 );
  bool login_monitor( );
  int monitor();
  void start_game( int u = 0 );
  void create_game( int u = 0 );
  void cancel_game( int u = 0 );
  void join_game( char name[NAME_SIZE], int u = 0 );
  int join_status( int u = 0 );
  void clear_join_status( int u = 0 );
  void leave_game( int u = 0 );
  void reject_player( char name[NAME_SIZE], int u = 0 );
  void send_chat_line( char line[CHAT_SIZE], int u = 0 );
  void send_packet( int command, char *info, int size, NODEADD node, int u );
  void send_packet_to_userlist( int command, char *info, int size, int u );
  bool get_packet( WNPACKET *wnpacket );
  int find_local_user_by_name( char name[NAME_SIZE] );
  USERNODE *find_node_by_name( char name[NAME_SIZE] );
  USERNODE *find_node_by_node( NODEADD node );
  USERNODE *find_node_by_game( NODEADD node, int ln, int u = 0 );
  USERNODE *find_node_by_zerotick();
  void add_node_to_list( USER *_user );
  void remove_node_from_list( USERNODE *node );
  USERNODE *first_node();
  USERNODE *next_node();
  int find_free_slot( USER *_user, int u = 0 );
  bool is_client_of( USER *other, int u = 0 );
  bool is_server_of( USER *other, int u = 0 );
  bool is_empty( NODEADD slot, int u = 0 );
  bool is_same( NODEADD node_one, NODEADD node_two );
  void add_chat_line( char data[CHAT_SIZE], USERNODE *from );
  bool get_chat_line( CHATLINE *data );
};

#endif
